#ifndef SNAKE_H
#define SNAKE_H

#include <config.h>
#include <stdio.h>
#include <snake_compat.h>
#include <snake_limits.h>


/* version string */
#define VERSION "0.16"


/* delay range */
#define MINDELAY 0
#define MAXDELAY 500

/* Direction type */
typedef int DIRECTION;

/*
 *
 * Opposite directions MUST BE both odd or even
 */
#define LEFT	0		/* left direction */
#define UP  	1		/* up direction */
#define RIGHT	2		/* right direction */
#define DOWN	3		/* down direction */

#define STOP	4		/* no direction (stopped) */

#define IS_VDIR(x) (x%2)	/* vertical direction */
#define IS_HDIR(x) (!(x%2))	/* horizontal direction */

#define IS_HORTOGONAL(x,y) ( (x%2) != (y%2) )
#define IS_OPPOSITE(x,y) (abs((x)-(y)) == 2)

/* Loop on directions */
#define LOOPDIRECTION(x) for ((x)=0; (x)<4; (x)++)

/* macros */
#define BE_FRUIT(x) ((x)|A_BOLD)
#define BE_SNAKE(x) ((x)|A_REVERSE)

#define IS_FRUIT(x) ( (x) ? ((x)->type == FRUIT) : 0 )
#define IS_SNAKE(x) ( (x) ? ((x)->type == SNAKE) : 0 )
#define IS_EXSNAKE(x) ( (x) ? ((x)->type == EXSNAKE) : 0 )

/* #define FRUITVAL(x) (((x)&A_CHARTEXT)-48)	* fruit value */
#define FRUITVAL(x) (((x)->value)-48)

/* get sprite pointer @ given coordinate */
#define GETSPRITE(a,y,x) *((a)->board+((y)*(a)->cols+(x)))

#define ARENAGET(y,x) arenaget((y),(x))
#define ARENAPUT(y,x,z) arenaput((y), (x), (z))

/* cell */
typedef struct t_cell
{
  int x;
  int y;
}
cell;

/* chain of points */
typedef struct t_snakebody
{
  int type;
  struct t_snake *self;
  struct t_cell cell;
  struct t_snakebody *next;
  struct t_snakebody *prev;
}
snakebody;

/* snake structure */
typedef struct t_snake
{
  int playernum;		/* player number */
  char name[MAXNAMLEN];		/* snake's name */
  int look;			/* snake's look */
  snakebody *head;		/* snake's head chain */
  snakebody *tail;		/* snake's tail chain */
  int length;			/* snake's length */
  int grow;			/* snake's growing state */
  int died;			/* did this snake die? */
  int dir;			/* snake's movement direction */
  unsigned long int score;	/* score */
  DIRECTION (*play) (struct t_snake *);		/* decision taker */
  void (*kill) (struct t_snake *);		/* killer */
  void (*destroy) (struct t_snake *);		/* destroier */
  void *brain;			/* snake's brain */
}
snake;

/* fruit structure */
typedef struct t_fruit
{
  int type;
  struct t_cell cell;
  int value;
}
fruit;

/* MESSAGES ************************************/
extern void message (char *format, ...);

/* ARENA ***************************************/
/* sprite types */
#define SNAKE 1
#define EXSNAKE 2
#define FRUIT 3
#define WALL 4

/* sprite template structure */
typedef struct t_sprite
{
  int type;
  char data[1]; /* should this be (void *) ? */
}
sprite;

/* Arena structure */
typedef struct arena_t
{
  int lines;        /* dimensions of the "bidimensional" sprite array */
  int cols;                               

  int init;         /* integrity check flag */
  sprite **board;   /* pointer to the sprite array */
} ARENA;



/* EXTERNAL FUNCTIONS */
extern int arenainit (void);
extern int arenalines (void);
extern int arenacols (void);
extern sprite *arenaget (int y, int x);
extern int arenaput (int y, int x, sprite * sp);

/* UI API */
extern void ui_drawmessage (char *);
extern void ui_drawarena (ARENA *);
extern void ui_updatescreen (void);
extern char *ui_prompt (char *);
extern int ui_getkey (void);

extern void finish (void);
extern int rand_between (int, int);
extern void *safe_malloc (size_t size, char *);
extern void *safe_calloc (size_t nmemb, size_t size, char *);
extern void *safe_realloc (void *ptr, size_t size, char *);
void rewindtypein (void);

extern snake *snake_make (void);
extern void snake_init (snake *);
extern int register_player (snake *);
extern void snake_reset (snake *);
extern void snake_kill (snake *);
extern void snake_trim (snake *);

extern void move_cell (cell *, DIRECTION, int);
extern int cell_distance (cell *, cell *);
extern void init_fruit (void);

extern int register_command (char *, int (*)(char *), char *);
extern int parse_command (char *);
extern int fetchtypein (void);

extern char *find_file (char *);

/* EXTERNAL DATA */
extern int delay;
extern snake *zoo[];
extern char *cmdfifofile;
extern FILE *cmdfifo;
extern int screen_initialized;
extern ARENA arena;
extern fruit frt;
extern char path[];


/* arena operations */
#define ARENA_HEIGHT (arena.lines)
#define ARENA_WIDTH (arena.cols)

#endif
